""" fleks.app
"""

from rich.align import Align  # noqa
from rich.console import Console  # noqa
from rich.console import Theme  # noqa
from rich.emoji import Emoji  # noqa
from rich.markdown import Markdown  # noqa
from rich.panel import Panel  # noqa
from rich.style import Style  # noqa
from rich.syntax import Syntax  # noqa
from rich.table import Table  # noqa
from rich.text import Text  # noqa


class AppBase:
    pass
